<?
/*
	System configuration stuff for LDAP
*/

include_once ("../proc.php");


//connects a server and tests if settings are ok
function TestServer($server,$auth,$passwd,$search=1)
{
	global $Set;
	$ldap_conn=ldap_connect($server);
	ldap_set_option($ldap_conn, LDAP_OPT_PROTOCOL_VERSION, 3);

	//first test if the server is reacable, since ldap apicalls
	//hang indefinitly!!
	$err=TestConnect($server,389);
	if ($err)
		return ($err);

	if (!ldap_bind($ldap_conn, $auth, $passwd))
	{
		return (ldap_error($ldap_conn));
	}
	if($search && !ldap_search($ldap_conn, $Set['LDAP_DN'], $Set['LDAP_DN']))
	{
		return(ldap_error($ldap_conn));
	}
	return false;
}

/* Update neccesary config files to refelect ldap settings
   Normally you call ApplyLDAPsettings, which also restarts stuff and cleans up the slave db.
*/
function StoreLDAPSettings()
{
	global $Set;
	// modify /etc/hosts..
	// If LDAP_SLAVE_SERVER is emtpy, we use the same ip as the MASTER_SERVER, to prevent
	// annoying timeouts from some services. (like slapd and smbd)
	FileReplaceLine("/etc/hosts","/ldap-master/","$Set[LDAP_MASTER_SERVER]\t\tldap-master");
	if ($Set[LDAP_SLAVE_SERVER])
		FileReplaceLine("/etc/hosts","/ldap-slave/","$Set[LDAP_SLAVE_SERVER]\t\tldap-slave");
	else
		FileReplaceLine("/etc/hosts","/ldap-slave/","$Set[LDAP_MASTER_SERVER]\t\tldap-slave");

	//update smbldap tools password
	FileReplaceLine("/etc/smbldap-tools/smbldap_bind.conf",
			"/slavePw/",
			'slavePw="'.$Set[LDAP_SLAVE_PASSWD].'"');

	FileReplaceLine("/etc/smbldap-tools/smbldap_bind.conf",
			"/masterPw/",
			'masterPw="'.$Set[LDAP_MASTER_PASSWD].'"');


	//SET the master and slave passwords in the appropriate slapd config files
	FileReplaceLine("/etc/openldap/slapd.conf.master","/^rootpw/","rootpw\t\"$Set[LDAP_MASTER_PASSWD]\"");
	FileReplaceLine("/etc/openldap/slapd.conf.mirror1","/^rootpw/","rootpw\t\"$Set[LDAP_MASTER_PASSWD]\"");
	FileReplaceLine("/etc/openldap/slapd.conf.mirror2","/^rootpw/","rootpw\t\"$Set[LDAP_SLAVE_PASSWD]\"");

	//SUPPLY the master and slave passwords, so the mirrors can connect eachother.
	FileReplaceLine("/etc/openldap/slapd.conf.mirror1","/^\s*credentials=/","\tcredentials=\"$Set[LDAP_SLAVE_PASSWD]\"");
	FileReplaceLine("/etc/openldap/slapd.conf.mirror2","/^\s*credentials=/","\tcredentials=\"$Set[LDAP_MASTER_PASSWD]\"");

	// point the ldap.conf symlink to the correct configfile.
	if ($Set[LDAP_MODE]=="master")
		FileSymlinkReplace("/etc/openldap/slapd.conf.master","/etc/openldap/slapd.conf");
	elseif ($Set[LDAP_MODE]=="mirror1")
		FileSymlinkReplace("/etc/openldap/slapd.conf.mirror1","/etc/openldap/slapd.conf");
	elseif ($Set[LDAP_MODE]=="mirror2")
		FileSymlinkReplace("/etc/openldap/slapd.conf.mirror2","/etc/openldap/slapd.conf");
	else
		FileDelete("/etc/openldap/slapd.conf");

}

//modify system configuration here
function ApplyLDAPsettings()
{
	global $Set;

	LogWarn(_ChangingLDAPSettings());

	//shutdown ldap first
	SvcDown("slapd");

	//store the new config settings
	StoreLDAPSettings();

	// empty the slave database
	SafeExec("rm /home/system/openldap/openldap-slave/* ; exit 0");

	//restart ldapd if neccesary
	if ($Set[LDAP_MODE] != "none")
		SvcUp("slapd");

	//Restart all services
	SvcRestartAll();

}


if ($_SERVER['argv'][1]=="init")
    StoreLDAPSettings();
